@@ -34,6 +34,42 @@ void serialize(Archive &ar, amici::Model &m, unsigned int version);
 
 namespace amici {
 
+/**
+ * @brief Describes the various model quantities
+ */
+enum class ModelQuantity {
+    J,
+    JB,
+    Jv,
+    JvB,
+    JDiag,
+    sx,
+    sy,
+    ssigmay,
+    xdot,
+    sxdot,
+    xBdot,
+    x0_rdata,
+    x0,
+    x_rdata,
+    x,
+    dwdw,
+    dwdx,
+    dwdp,
+    y,
+    dydp,
+    dydx,
+    w,
+    root,
+    qBdot,
+    qBdot_ss,
+    xBdot_ss,
+    JSparseB_ss,
+};
+
+extern const std::map<ModelQuantity, std::string> model_quantity_to_str;
+
+
 /**
  * @brief The Model class represents an AMICI ODE/DAE model.
  *
@@ -61,7 +97,8 @@ class Model : public AbstractModel, public ModelDimensions {
           SimulationParameters simulation_parameters,
           amici::SecondOrderMode o2mode,
           std::vector<amici::realtype> idlist,
-          std::vector<int> z2event, bool pythonGenerated = false,
+          std::vector<int> z2event,
+          bool pythonGenerated = false,
           int ndxdotdp_explicit = 0, int ndxdotdx_explicit = 0,
           int w_recursion_depth = 0);
 
@@ -166,9 +203,11 @@ class Model : public AbstractModel, public ModelDimensions {
      * @param sdx Reference to time derivative of state sensitivities (DAE only)
      * @param computeSensitivities Flag indicating whether sensitivities are to
      * be computed
+     * @param roots_found boolean indicators indicating whether roots were found at t0 by this fun
      */
     void initialize(AmiVector &x, AmiVector &dx, AmiVectorArray &sx,
-                    AmiVectorArray &sdx, bool computeSensitivities);
+                    AmiVectorArray &sdx, bool computeSensitivities,
+                    std::vector<int> &roots_found);
 
     /**
      * @brief Initialize model properties.
@@ -200,8 +239,10 @@ class Model : public AbstractModel, public ModelDimensions {
      *
      * @param x Reference to state variables
      * @param dx Reference to time derivative of states (DAE only)
+     * @param roots_found boolean indicators indicating whether roots were found at t0 by this fun
      */
-    void initHeaviside(const AmiVector &x, const AmiVector &dx);
+    void initEvents(const AmiVector &x, const AmiVector &dx,
+                    std::vector<int> &roots_found);
 
     /**
      * @brief Get number of parameters wrt to which sensitivities are computed.
@@ -1201,18 +1242,43 @@ class Model : public AbstractModel, public ModelDimensions {
      */
     void updateHeavisideB(const int *rootsfound);
 
+
     /**
      * @brief Check if the given array has only finite elements.
      *
-     * If not, try to give hints by which other fields this could be caused.
+     * For (1D) spans.
      *
-     * @param array Array to check
-     * @param fun Name of the function that generated the values (for more
-     * informative messages).
-     * @return `amici::AMICI_RECOVERABLE_ERROR` if a NaN/Inf value was found,
-     * `amici::AMICI_SUCCESS` otherwise
+     * @param array
+     * @param model_quantity The model quantity `array` corresponds to
+     * @return
      */
-    int checkFinite(gsl::span<const realtype> array, const char *fun) const;
+    int checkFinite(gsl::span<const realtype> array,
+                           ModelQuantity model_quantity) const;
+    /**
+     * @brief Check if the given array has only finite elements.
+     *
+     * For flattened 2D arrays.
+     *
+     * @param array Flattened matrix
+     * @param model_quantity The model quantity `array` corresponds to
+     * @param num_cols Number of columns of the non-flattened matrix
+     * @return
+     */
+    int checkFinite(gsl::span<const realtype> array,
+                    ModelQuantity model_quantity,
+                    size_t num_cols) const;
+
+    /**
+     * @brief Check if the given array has only finite elements.
+     *
+     * For SUNMatrix.
+     *
+     * @param m Matrix to check
+     * @param model_quantity The model quantity `m` corresponds to
+     * @param t current timepoint
+     * @return
+     */
+    int checkFinite(SUNMatrix m, ModelQuantity model_quantity, realtype t) const;
 
     /**
      * @brief Set whether the result of every call to `Model::f*` should be
@@ -1803,6 +1869,11 @@ class Model : public AbstractModel, public ModelDimensions {
     /** vector of bools indicating whether state variables are to be assumed to
      * be positive */
     std::vector<bool> state_is_non_negative_;
+    
+    /** Vector of booleans indicating the initial boolean value for every event trigger function. Events at t0
+     * can only trigger if the initial value is set to `false`. Must be specified during model compilation by
+     * setting the `initialValue` attribute of an event trigger. */
+    std::vector<bool> root_initial_values_;
 
     /** boolean indicating whether any entry in stateIsNonNegative is `true` */
     bool any_state_non_negative_ {false};
