@@ -178,4 +178,9 @@ std::string printfToString(const char *fmt, va_list ap) {
     return str;
 }
 
+std::pair<size_t, size_t> unravel_index(size_t flat_idx, size_t num_cols)
+{
+    return {flat_idx / num_cols, flat_idx % num_cols};
+}
+
 } // namespace amici
