@@ -15,6 +15,39 @@
 
 namespace amici {
 
+/**
+ * @brief Maps ModelQuantity items to their string value
+ */
+const std::map<ModelQuantity, std::string> model_quantity_to_str {
+    {ModelQuantity::J, "J"},
+    {ModelQuantity::JB, "JB"},
+    {ModelQuantity::Jv, "Jv"},
+    {ModelQuantity::JvB, "JvB"},
+    {ModelQuantity::JDiag, "JDiag"},
+    {ModelQuantity::sx, "sx"},
+    {ModelQuantity::sy, "sy"},
+    {ModelQuantity::ssigmay, "ssigmay"},
+    {ModelQuantity::xdot, "xdot"},
+    {ModelQuantity::sxdot, "sxdot"},
+    {ModelQuantity::xBdot, "xBdot"},
+    {ModelQuantity::x0, "x0"},
+    {ModelQuantity::x0_rdata, "x0_rdata"},
+    {ModelQuantity::x, "x"},
+    {ModelQuantity::x_rdata, "x_rdata"},
+    {ModelQuantity::dwdw, "dwdw"},
+    {ModelQuantity::dwdx, "dwdx"},
+    {ModelQuantity::dwdp, "dwdp"},
+    {ModelQuantity::y, "y"},
+    {ModelQuantity::dydp, "dydp"},
+    {ModelQuantity::dydx, "dydx"},
+    {ModelQuantity::w, "w"},
+    {ModelQuantity::root, "root"},
+    {ModelQuantity::qBdot, "qBdot"},
+    {ModelQuantity::qBdot_ss, "qBdot_ss"},
+    {ModelQuantity::xBdot_ss, "xBdot_ss"},
+    {ModelQuantity::JSparseB_ss, "JSparseB_ss"},
+};
+
 /**
  * @brief local helper function to get parameters
  * @param ids vector of name/ids of (fixed)Parameters
@@ -97,7 +130,8 @@ static int setValueByIdRegex(std::vector<std::string> const &ids,
 
 Model::Model(ModelDimensions const & model_dimensions,
              SimulationParameters simulation_parameters,
-             SecondOrderMode o2mode, std::vector<realtype> idlist, std::vector<int> z2event,
+             SecondOrderMode o2mode, std::vector<realtype> idlist,
+             std::vector<int> z2event,
              const bool pythonGenerated, const int ndxdotdp_explicit,
              const int ndxdotdx_explicit, const int w_recursion_depth)
     : ModelDimensions(model_dimensions), pythonGenerated(pythonGenerated),
@@ -120,6 +154,8 @@ Model::Model(ModelDimensions const & model_dimensions,
                       simulation_parameters_.pscale, state_.unscaledParameters);
     state_.fixedParameters = simulation_parameters_.fixedParameters;
     state_.plist = simulation_parameters_.plist;
+          
+    root_initial_values_.resize(ne, true);
 
     /* If Matlab wrapped: dxdotdp is a full AmiVector,
        if Python wrapped: dxdotdp_explicit and dxdotdp_implicit are CSC matrices
@@ -208,7 +244,8 @@ bool operator==(const ModelDimensions &a, const ModelDimensions &b) {
 
 
 void Model::initialize(AmiVector &x, AmiVector &dx, AmiVectorArray &sx,
-                       AmiVectorArray & /*sdx*/, bool computeSensitivities) {
+                       AmiVectorArray & /*sdx*/, bool computeSensitivities,
+                       std::vector<int> &roots_found) {
     initializeStates(x);
     if (computeSensitivities)
         initializeStateSensitivities(sx, x);
@@ -218,7 +255,7 @@ void Model::initialize(AmiVector &x, AmiVector &dx, AmiVectorArray &sx,
         fsdx0();
 
     if (ne)
-        initHeaviside(x, dx);
+        initEvents(x, dx, roots_found);
 }
 
 void Model::initializeB(AmiVector &xB, AmiVector &dxB, AmiVector &xQB,
@@ -265,14 +302,18 @@ void Model::initializeStateSensitivities(AmiVectorArray &sx,
     }
 }
 
-void Model::initHeaviside(AmiVector const &x, AmiVector const &dx) {
+void Model::initEvents(AmiVector const &x, AmiVector const &dx,
+                       std::vector<int> &roots_found) {
     std::vector<realtype> rootvals(ne, 0.0);
     froot(simulation_parameters_.tstart_, x, dx, rootvals);
+    std::fill(roots_found.begin(), roots_found.end(), 0);
     for (int ie = 0; ie < ne; ie++) {
         if (rootvals.at(ie) < 0) {
             state_.h.at(ie) = 0.0;
         } else {
             state_.h.at(ie) = 1.0;
+            if (!root_initial_values_.at(ie)) // only false->true triggers event
+                roots_found.at(ie) = 1;
         }
     }
 }
@@ -843,7 +884,7 @@ void Model::getObservableSensitivity(gsl::span<realtype> sy, const realtype t,
     writeSlice(derived_state_.dydp_, sy);
 
     if (always_check_finite_)
-        checkFinite(sy, "sy");
+        checkFinite(sy, ModelQuantity::sy, nplist());
 }
 
 void Model::getObservableSigma(gsl::span<realtype> sigmay, const int it,
@@ -875,7 +916,7 @@ void Model::getObservableSigmaSensitivity(gsl::span<realtype> ssigmay,
     }
 
     if (always_check_finite_)
-        checkFinite(ssigmay, "ssigmay");
+        checkFinite(ssigmay, ModelQuantity::ssigmay, nplist());
 }
 
 void Model::addObservableObjective(realtype &Jy, const int it,
@@ -1218,18 +1259,242 @@ void Model::updateHeavisideB(const int *rootsfound) {
     }
 }
 
-
-int Model::checkFinite(gsl::span<const realtype> array, const char *fun) const {
-    auto result = app->checkFinite(array, fun);
-
-    if (result != AMICI_SUCCESS) {
-        app->checkFinite(state_.fixedParameters, "k");
-        app->checkFinite(state_.unscaledParameters, "p");
-        app->checkFinite(derived_state_.w_, "w");
-        app->checkFinite(simulation_parameters_.ts_, "t");
+int Model::checkFinite(gsl::span<const realtype> array,
+                       ModelQuantity model_quantity) const
+{
+    auto it = std::find_if(array.begin(), array.end(),
+                           [](realtype x){return !std::isfinite(x);});
+    if(it == array.end()) {
+        return AMICI_SUCCESS;
     }
 
-    return result;
+    // there is some issue - produce a meaningful message
+    auto flat_index = it - array.begin();
+
+    std::string msg_id;
+    std::string non_finite_type;
+    if (std::isnan(array[flat_index])) {
+        msg_id = "AMICI:NaN";
+        non_finite_type = "NaN";
+    } else if (std::isinf(array[flat_index])) {
+        msg_id = "AMICI:Inf";
+        non_finite_type = "Inf";
+    }
+    std::string element_id = std::to_string(flat_index);
+
+    switch (model_quantity) {
+    case ModelQuantity::xdot:
+    case ModelQuantity::xBdot:
+    case ModelQuantity::x0:
+    case ModelQuantity::x:
+    case ModelQuantity::x_rdata:
+    case ModelQuantity::x0_rdata:
+    case ModelQuantity::Jv:
+    case ModelQuantity::JvB:
+    case ModelQuantity::JDiag:
+        if(hasStateIds()) {
+            element_id = getStateIdsSolver()[flat_index];
+        }
+        break;
+    case ModelQuantity::y:
+        if(hasObservableIds()) {
+            element_id = getObservableIds()[flat_index];
+        }
+        break;
+    case ModelQuantity::w:
+        if(hasExpressionIds()) {
+            element_id = getExpressionIds()[flat_index];
+        }
+        break;
+    default:
+        break;
+    }
+
+    std::string model_quantity_str;
+    try {
+        model_quantity_str = model_quantity_to_str.at(model_quantity);
+    } catch (std::out_of_range const&) {
+        // Missing model quantity string - terminate if this is a debug build,
+        // but show the quantity number if non-debug.
+        gsl_ExpectsDebug(false);
+        model_quantity_str = std::to_string(static_cast<int>(model_quantity));
+    }
+    app->warningF(msg_id.c_str(),
+                  "AMICI encountered a %s value for %s[%i] (%s)",
+                  non_finite_type.c_str(),
+                  model_quantity_str.c_str(),
+                  static_cast<int>(flat_index),
+                  element_id.c_str()
+                  );
+
+    // check upstream
+    app->checkFinite(state_.fixedParameters, "k");
+    app->checkFinite(state_.unscaledParameters, "p");
+    app->checkFinite(simulation_parameters_.ts_, "t");
+    if(!always_check_finite_) {
+        // don't check twice if always_check_finite_ is true
+        app->checkFinite(derived_state_.w_, "w");
+    }
+
+    return AMICI_RECOVERABLE_ERROR;
+}
+
+int Model::checkFinite(gsl::span<const realtype> array,
+                       ModelQuantity model_quantity, size_t num_cols) const
+{
+    auto it = std::find_if(array.begin(), array.end(),
+                           [](realtype x){return !std::isfinite(x);});
+    if(it == array.end()) {
+        return AMICI_SUCCESS;
+    }
+
+    // there is some issue - produce a meaningful message
+    auto flat_index = it - array.begin();
+    sunindextype row, col;
+    std::tie(row, col) = unravel_index(flat_index, num_cols);
+    std::string msg_id;
+    std::string non_finite_type;
+    if (std::isnan(array[flat_index])) {
+        msg_id = "AMICI:NaN";
+        non_finite_type = "NaN";
+    } else if (std::isinf(array[flat_index])) {
+        msg_id = "AMICI:Inf";
+        non_finite_type = "Inf";
+    }
+    std::string row_id = std::to_string(row);
+    std::string col_id = std::to_string(col);
+
+    switch (model_quantity) {
+    case ModelQuantity::sy:
+    case ModelQuantity::ssigmay:
+    case ModelQuantity::dydp:
+        row_id += " " + getObservableIds()[row];
+        col_id += " " + getParameterIds()[plist(col)];
+        break;
+    case ModelQuantity::dydx:
+        row_id += " " + getObservableIds()[row];
+        col_id += " " + getStateIdsSolver()[col];
+        break;
+    default:
+        break;
+    }
+
+    std::string model_quantity_str;
+    try {
+        model_quantity_str = model_quantity_to_str.at(model_quantity);
+    } catch (std::out_of_range const&) {
+        // Missing model quantity string - terminate if this is a debug build,
+        // but show the quantity number if non-debug.
+        gsl_ExpectsDebug(false);
+        model_quantity_str = std::to_string(static_cast<int>(model_quantity));
+    }
+
+    app->warningF(msg_id.c_str(),
+                  "AMICI encountered a %s value for %s[%i] (%s, %s)",
+                  non_finite_type.c_str(),
+                  model_quantity_str.c_str(),
+                  static_cast<int>(flat_index),
+                  row_id.c_str(),
+                  col_id.c_str()
+                  );
+
+    // check upstream
+    app->checkFinite(state_.fixedParameters, "k");
+    app->checkFinite(state_.unscaledParameters, "p");
+    app->checkFinite(simulation_parameters_.ts_, "t");
+    app->checkFinite(derived_state_.w_, "w");
+
+    return AMICI_RECOVERABLE_ERROR;
+}
+
+int Model::checkFinite(SUNMatrix m, ModelQuantity model_quantity, realtype t) const
+{
+    // check flat array, to see if there are any issues
+    // (faster, in particular for sparse arrays)
+    auto m_flat = gsl::make_span(m);
+    auto it = std::find_if(m_flat.begin(), m_flat.end(),
+                    [](realtype x){return !std::isfinite(x);});
+    if(it == m_flat.end()) {
+        return AMICI_SUCCESS;
+    }
+
+    // there is some issue - produce a meaningful message
+    auto flat_index = it - m_flat.begin();
+    sunindextype row, col;
+    std::tie(row, col) = unravel_index(flat_index, m);
+    std::string msg_id;
+    std::string non_finite_type;
+    if (std::isnan(m_flat[flat_index])) {
+        msg_id = "AMICI:NaN";
+        non_finite_type = "NaN";
+    } else if (std::isinf(m_flat[flat_index])) {
+        msg_id = "AMICI:Inf";
+        non_finite_type = "Inf";
+    } else {
+        throw std::runtime_error(
+            "Value is not finite, but neither infinite nor NaN.");
+    }
+    std::string row_id = std::to_string(row);
+    std::string col_id = std::to_string(col);
+
+    switch (model_quantity) {
+    case ModelQuantity::J:
+    case ModelQuantity::JB:
+        if(hasStateIds()) {
+            auto state_ids = getStateIdsSolver();
+            row_id += " " + state_ids[row];
+            col_id += " " + state_ids[col];
+        }
+        break;
+    case ModelQuantity::dwdx:
+        if(hasExpressionIds())
+            row_id += " " + getExpressionIds()[row];
+        if(hasStateIds())
+            col_id += " " + getStateIdsSolver()[col];
+        break;
+    case ModelQuantity::dwdw:
+        if(hasExpressionIds()) {
+            auto expr_ids = getExpressionIds();
+            row_id += " " + expr_ids[row];
+            col_id += " " + expr_ids[col];
+        }
+        break;
+    case ModelQuantity::dwdp:
+        if(hasExpressionIds())
+            row_id += " " + getExpressionIds()[row];
+        if(hasParameterIds())
+            col_id += " " + getParameterIds()[plist(col)];
+        break;
+    default:
+        break;
+    }
+
+    std::string model_quantity_str;
+    try {
+        model_quantity_str = model_quantity_to_str.at(model_quantity);
+    } catch (std::out_of_range const&) {
+        // Missing model quantity string - terminate if this is a debug build,
+        // but show the quantity number if non-debug.
+        gsl_ExpectsDebug(false);
+        model_quantity_str = std::to_string(static_cast<int>(model_quantity));
+    }
+    app->warningF(msg_id.c_str(),
+                  "AMICI encountered a %s value for %s[%i] (%s, %s) at t=%g",
+                  non_finite_type.c_str(),
+                  model_quantity_str.c_str(),
+                  static_cast<int>(flat_index),
+                  row_id.c_str(),
+                  col_id.c_str(),
+                  t
+                  );
+
+    // check upstream
+    app->checkFinite(state_.fixedParameters, "k");
+    app->checkFinite(state_.unscaledParameters, "p");
+    app->checkFinite(simulation_parameters_.ts_, "t");
+    app->checkFinite(derived_state_.w_, "w");
+
+    return AMICI_RECOVERABLE_ERROR;
 }
 
 void Model::setAlwaysCheckFinite(bool alwaysCheck) {
@@ -1250,8 +1515,8 @@ void Model::fx0(AmiVector &x) {
               state_.fixedParameters.data());
 
     if (always_check_finite_) {
-        checkFinite(derived_state_.x_rdata_, "x0 x_rdata");
-        checkFinite(x.getVector(), "x0 x");
+        checkFinite(derived_state_.x_rdata_, ModelQuantity::x0_rdata);
+        checkFinite(x.getVector(), ModelQuantity::x0);
     }
 }
 
@@ -1333,7 +1598,7 @@ void Model::fx_rdata(AmiVector &x_rdata, const AmiVector &x) {
     fx_rdata(x_rdata.data(), computeX_pos(x), state_.total_cl.data(),
              state_.unscaledParameters.data(), state_.fixedParameters.data());
     if (always_check_finite_)
-        checkFinite(x_rdata.getVector(), "x_rdata");
+        checkFinite(x_rdata.getVector(), ModelQuantity::x_rdata);
 }
 
 void Model::fsx_rdata(AmiVectorArray &sx_rdata, const AmiVectorArray &sx,
@@ -1411,7 +1676,7 @@ void Model::fy(const realtype t, const AmiVector &x) {
        state_.h.data(), derived_state_.w_.data());
 
     if (always_check_finite_) {
-        app->checkFinite(gsl::make_span(derived_state_.y_.data(), ny), "y");
+        checkFinite(gsl::make_span(derived_state_.y_.data(), ny), ModelQuantity::y);
     }
 }
 
@@ -1441,7 +1706,7 @@ void Model::fdydp(const realtype t, const AmiVector &x) {
         }
 
     if (always_check_finite_) {
-        app->checkFinite(derived_state_.dydp_, "dydp");
+        checkFinite(derived_state_.dydp_, ModelQuantity::dydp, nplist());
     }
 }
 
@@ -1461,7 +1726,7 @@ void Model::fdydx(const realtype t, const AmiVector &x) {
           derived_state_.dwdx_.data());
 
     if (always_check_finite_) {
-        app->checkFinite(derived_state_.dydx_, "dydx");
+        checkFinite(derived_state_.dydx_, ModelQuantity::dydx, ny);
     }
 }
 
@@ -2076,7 +2341,7 @@ void Model::fw(const realtype t, const realtype *x) {
        state_.fixedParameters.data(), state_.h.data(), state_.total_cl.data());
 
     if (always_check_finite_) {
-        app->checkFinite(derived_state_.w_, "w");
+        checkFinite(derived_state_.w_, ModelQuantity::w);
     }
 }
 
@@ -2116,7 +2381,7 @@ void Model::fdwdp(const realtype t, const realtype *x) {
     }
 
     if (always_check_finite_) {
-        app->checkFinite(gsl::make_span(derived_state_.dwdp_.get()), "dwdp");
+        checkFinite(derived_state_.dwdp_.get(), ModelQuantity::dwdp, t);
     }
 }
 
@@ -2157,7 +2422,7 @@ void Model::fdwdx(const realtype t, const realtype *x) {
     }
 
     if (always_check_finite_) {
-        app->checkFinite(gsl::make_span(derived_state_.dwdx_.get()), "dwdx");
+        checkFinite(derived_state_.dwdx_.get(), ModelQuantity::dwdx, t);
     }
 }
 
@@ -2172,7 +2437,7 @@ void Model::fdwdw(const realtype t, const realtype *x) {
           derived_state_.w_.data(), state_.total_cl.data());
 
     if (always_check_finite_) {
-        app->checkFinite(gsl::make_span(dwdw_.get()), "dwdw");
+        checkFinite(dwdw_.get(), ModelQuantity::dwdw, t);
     }
 }
 
