@@ -105,7 +105,7 @@ void Model_ODE::fJDiag(const realtype t, AmiVector &JDiag,
                        const realtype /*cj*/, const AmiVector &x,
                        const AmiVector & /*dx*/) {
     fJDiag(t, JDiag.getNVector(), x.getNVector());
-    if (checkFinite(JDiag.getVector(), "Jacobian") != AMICI_SUCCESS)
+    if (checkFinite(JDiag.getVector(), ModelQuantity::JDiag) != AMICI_SUCCESS)
         throw AmiException("Evaluation of fJDiag failed!");
 }
 
